using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Globalization;
using System.Collections.ObjectModel;

namespace TestMainWindow
{
    public class NcCalendar : ObservableCollection<NcCalendarWeek>, INotifyPropertyChanged
    {
        #region ֶζ
        private int mYear;
        private int mMonth;
        private NcCalendarDay mCurrent = NcCalendarDay.Empty;

        #endregion
        #region ¼
        public event EventHandler SelectedChanged;
        public event EventHandler CurrentChanged;
        #endregion
        #region 캯
        public NcCalendar() : this(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day) { }
        public NcCalendar(DateTime date) : this(date.Year, date.Month, date.Day) { }
        public NcCalendar(int year, int month) : this(year, month, 1) { }
        public NcCalendar(int year, int month, int day) { BuildCalendar(year, month, day); }
        #endregion
        #region Զ
        public int Year
        {
            get { return mYear; }
            set
            {
                int day = (mCurrent != null) ? day = mCurrent.Day : 1;
                if (BuildCalendar(value, mMonth, day)) OnPropertyChanged(new PropertyChangedEventArgs("Year"));
            }
        }
        public int Month
        {
            get { return mMonth; }
            set
            {
                int day = (mCurrent != null) ? day = mCurrent.Day : 1;
                if (BuildCalendar(mYear, value, day)) OnPropertyChanged(new PropertyChangedEventArgs("Month"));
            }
        }
        public int Day
        {
            get { return mCurrent.Day; }
            set { if (BuildCalendar(mYear, mMonth, value)) OnPropertyChanged(new PropertyChangedEventArgs("Day")); }
        }

        public NcCalendarDay Current
        {
            get { return mCurrent; }
            set
            {
                if (value == null || value == NcCalendarDay.Empty || value == mCurrent || value.Value.Date == mCurrent.Value.Date) return;
                bool setOK = false;
                foreach (NcCalendarWeek week in this)
                {
                    foreach (NcCalendarDay one in week)
                    {
                        if (one.IsEmpty) continue;
                        if (one.Value.Date == value.Value.Date)
                        {
                            mCurrent = one;
                            setOK = true;
                            OnPropertyChanged(new PropertyChangedEventArgs("Day"));
                            this.OnSelectedChanged(new EventArgs());
                            break;
                        }
                    }
                    if (setOK) break;
                }
            }
        }
        #endregion

        #region 
        public void SetValue(DateTime date) { BuildCalendar(date); }
        /// <summary>
        /// һ
        /// </summary>
        public void NextYear() 
        {
            if (mCurrent.Year == mYear && mCurrent.Month == mMonth)
            {
                BuildCalendar(mCurrent.Value.AddYears(1));
                return;
            }

            int newYear = mYear + 1;
            int newMonth = mMonth;
            int newDay = (mCurrent==null || mCurrent == NcCalendarDay.Empty) ? DateTime.Now.Day : mCurrent.Value.Day;

            DateTime maxDate = CultureInfo.CurrentCulture.Calendar.MaxSupportedDateTime.Date;
            if (newYear > maxDate.Year) return;

            int maxMonthOfYear = CultureInfo.CurrentCulture.Calendar.GetMonthsInYear(newYear);
            if (newMonth > maxMonthOfYear) newMonth = maxMonthOfYear;
            int maxDayOfMonth = CultureInfo.CurrentCulture.Calendar.GetDaysInMonth(newYear, newMonth);
            if (newDay > maxDayOfMonth) newDay = maxDayOfMonth;

            BuildCalendar(newYear, newMonth, newDay); 
        }
        /// <summary>
        /// ǰһ
        /// </summary>
        public void PreviousYear() 
        {
            if (mCurrent.Year == mYear && mCurrent.Month == mMonth)
            {
                BuildCalendar(mCurrent.Value.AddYears(-1));
                return;
            }

            int newYear = mYear - 1;
            int newMonth = mMonth;
            int newDay = (mCurrent == null || mCurrent == NcCalendarDay.Empty) ? DateTime.Now.Day : mCurrent.Value.Day;

            DateTime minDate = CultureInfo.CurrentCulture.Calendar.MinSupportedDateTime.Date;
            if (newYear < minDate.Year) return;

            int maxMonthOfYear = CultureInfo.CurrentCulture.Calendar.GetMonthsInYear(newYear);
            if (newMonth > maxMonthOfYear) newMonth = maxMonthOfYear;
            int maxDayOfMonth = CultureInfo.CurrentCulture.Calendar.GetDaysInMonth(newYear, newMonth);
            if (newDay > maxDayOfMonth) newDay = maxDayOfMonth;

            BuildCalendar(newYear, newMonth, newDay);
        }
        /// <summary>
        /// һ
        /// </summary>
        public void NextMonth() 
        {
            if (mCurrent.Year ==mYear && mCurrent.Month == mMonth)
            {
                BuildCalendar(mCurrent.Value.AddMonths(1));
                return;
            }
            int newYear = mYear;
            int newMonth = mMonth + 1;
            int newDay = (mCurrent == null || mCurrent == NcCalendarDay.Empty) ? DateTime.Now.Day : mCurrent.Value.Day;

            int maxMonthOfYear = CultureInfo.CurrentCulture.Calendar.GetMonthsInYear(newYear);
            if (newMonth > maxMonthOfYear)
            {
                newYear++;
                DateTime maxDate = CultureInfo.CurrentCulture.Calendar.MaxSupportedDateTime.Date;
                if (newYear > maxDate.Year)
                    return;
                newMonth = 1;
            }

            int maxDayOfMonth = CultureInfo.CurrentCulture.Calendar.GetDaysInMonth(newYear, newMonth);
            if (newDay > maxDayOfMonth) newDay = maxDayOfMonth;

            BuildCalendar(newYear, newMonth, newDay);
        }
        /// <summary>
        /// ǰһ
        /// </summary>
        public void PreviousMonth() 
        {
            if (mCurrent.Year == mYear && mCurrent.Month == mMonth)
            {
                BuildCalendar(mCurrent.Value.AddMonths(-1));
                return;
            }

            int newYear = mYear;
            int newMonth = mMonth - 1;
            int newDay = (mCurrent == null || mCurrent == NcCalendarDay.Empty) ? DateTime.Now.Day : mCurrent.Value.Day;

            DateTime minDate = CultureInfo.CurrentCulture.Calendar.MinSupportedDateTime.Date;
            if (newMonth < 1)
            {
                newYear = newYear - 1;
                if (newYear < minDate.Year) return;
                newMonth = CultureInfo.CurrentCulture.Calendar.GetMonthsInYear(newYear);
            }

            int maxDayOfMonth = CultureInfo.CurrentCulture.Calendar.GetDaysInMonth(newYear, newMonth);
            if (newDay > maxDayOfMonth) newDay = maxDayOfMonth;

            BuildCalendar(newYear, newMonth, newDay);

        }
        /// <summary>
        /// ת
        /// </summary>
        public void Today() { BuildCalendar(DateTime.Now.Date); }
        /// <summary>
        /// áǰա
        /// </summary>
        /// <param name="day"></param>
        /// <returns></returns>
        private bool setDay(int day)
        {
            foreach (NcCalendarWeek week in this)
            {
                foreach (NcCalendarDay one in week)
                {
                    if (!one.IsEmpty && one.Day == day)
                    {
                        if (mCurrent != null) mCurrent.IsCurrent = false;
                        mCurrent = one;
                        OnPropertyChanged(new PropertyChangedEventArgs("Current"));
                        OnPropertyChanged(new PropertyChangedEventArgs("Day"));
                        one.OnPropertyChanged("IsCurrent");
                        this.OnSelectedChanged(new EventArgs());
                        return true;
                    }
                    else
                    {
                        mCurrent = null;
                        this.OnPropertyChanged(new PropertyChangedEventArgs("IsCurrent"));
                    }
                }
            }
            return false;
        }

        private bool BuildCalendar(DateTime date) { if (date != null)  return BuildCalendar(date.Year, date.Month, date.Day); }
        /// <summary>
        /// ͼ
        /// </summary>
        /// <param name="year"></param>
        /// <param name="month"></param>
        /// <param name="day"></param>
        /// <returns></returns>
        private bool BuildCalendar(int year, int month, int day)
        {
            if (mCurrent != null)
            {
                // ꡢ¡վûб仯κθı
                if (year == mCurrent.Year && month == mCurrent.Month && day == mCurrent.Day) return true;
                // ꡢûб仯Ҫؽбλǰ
                if (year == mCurrent.Year && month == mCurrent.Month) return setDay(day);
            }

            DateTime minDate = CultureInfo.CurrentCulture.Calendar.MinSupportedDateTime.Date;
            DateTime maxDate = CultureInfo.CurrentCulture.Calendar.MaxSupportedDateTime.Date;
            int maxMonthOfYear = CultureInfo.CurrentCulture.Calendar.GetMonthsInYear(year);


            // 1ղϷΧ޶ʱFalse
            if (year < minDate.Year || year > maxDate.Year || month < 1 || month > maxMonthOfYear) return false;

            // 2ղϷΧ޶ʱ淶Χ
            //if (year < minDate.Year) year = minDate.Year;
            //if (year > maxDate.Year) year = maxDate.Year;
            //if (month < 1) month = 1;
            //if (month > maxMonthOfYear) month = maxMonthOfYear;


            DateTime firstDay = new DateTime(year, month, 1);
            int emptyDays = (int)firstDay.DayOfWeek;

            int days = CultureInfo.CurrentCulture.Calendar.GetDaysInMonth(year, month);
            int weeks = (int)Math.Ceiling(((float)days) / 7);

            if (day < 1 || day > days) day = (mCurrent != null) ? mCurrent.Day : 1;

            this.Clear();

            int dayOfWeekCount = 0;
            int weekCount = 0;
            NcCalendarWeek week = new NcCalendarWeek();
            NcCalendarDay item = NcCalendarDay.Empty;
            if (emptyDays > 0)
            {
                for (int i = emptyDays; i > 0; i--)
                {
                    item = new NcCalendarDay(firstDay.AddDays(-i));
                    item.Owner = this;
                    item.IsPreviousMonth = true;
                    week.Add(item);  //"ϸ"
                    dayOfWeekCount++;
                }
            }

            NcCalendarDay currentDayTo = null;
            for (int i = 1; i <= days; i++)
            {
                item = new NcCalendarDay(new DateTime(year, month, i));
                item.Owner = this;
                week.Add(item);
                dayOfWeekCount++;
                if (dayOfWeekCount >= 7)
                {
                    dayOfWeekCount = 0;
                    weekCount++;
                    this.Add(week);
                    week = new NcCalendarWeek();
                }
                if (i == day) currentDayTo = item;
            }


            firstDay = item.Value.Date;    // firstDay
            if (dayOfWeekCount > 0)
            {
                for (int i = dayOfWeekCount; i < 7; i++)
                {
                    item = new NcCalendarDay(firstDay.AddDays(i - dayOfWeekCount + 1));
                    item.IsNextMonth = true;
                    item.Owner = this;
                    week.Add(item);  //"¸"
                }
                if (dayOfWeekCount < 7) this.Add(week);
            }

            //if (this.mCurrent == null || this.mCurrent != currentDayTo)
            if (this.mCurrent == null)
            {
                this.mCurrent = currentDayTo;
                this.OnSelectedChanged(new EventArgs());
            }

            mYear = year;
            mMonth = month;

            OnPropertyChanged(new PropertyChangedEventArgs("Day"));
            OnPropertyChanged(new PropertyChangedEventArgs("Month"));
            OnPropertyChanged(new PropertyChangedEventArgs("Year"));

            return true;
        }
        #endregion

        protected internal virtual void OnSelectedChanged(EventArgs e) { if (this.SelectedChanged != null) this.SelectedChanged(this, e); }
        protected internal virtual void OnCurrentChanged(EventArgs e) { if (this.CurrentChanged != null) this.CurrentChanged(this, e); }
    }
    public class NcCalendarWeek : ObservableCollection<NcCalendarDay>, INotifyPropertyChanged { }

}
